/* 
options.c 
ALICE V 2.0, January 1993
library functions for option handling 
(C) Ralf Kaufmann 1993
used C compiler: Pure C 
*/ 
 
/* Includefiles */ 
 
#define __GEMLIB_OLDNAMES
#include <gem.h> 
#include <stdio.h> 
#include <stdlib.h> 
#include "language.h" 
#include "prvdef.h" 
#include "mewidi.h" 
#include "messages.h" 
#include "scancode.h" 
#include "structs.h" 
#include "rsc.h" 

// Funktionsprototypen
void display_pos(int w_index);
void display_time(short flag); 
short handle_dial(OBJECT *dial_tree, short start_obj); 
void put_message(int msg_nr);
void refresh_window(int w_handle);
 
/* References to global data defined elsewhere  */ 
 
extern  WIND_DATA windows[]; 
extern  BUFFER    buffers[]; 
extern  OBJECT    *menu_tree; 
extern  OBJECT    *dial_about, *dial_help, *dial_print; 
extern  OBJECT    *dial_search, *dial_replace, *dial_gotoli; 
extern  OBJECT    *dial_info, *dial_selebuf, *dial_options; 
extern  OBJECT    *dial_repl_info, *dial_gotopa, *dial_helpfi;
extern  OBJECT    *dial_helpms, *dial_helpbl, *dial_helpwi;
extern  OBJECT    *dial_pinfo;

extern  long    param_line; 
extern  int     page_length, left_margin; 
extern  int     tab_size, backup, clocky, header, lines;
extern  int     showcr, showpos; 
extern  int     buffer_index, line_len, eol_warn; 
extern  int     from_page, to_page, page_offset, closeall;
extern  int     sea_direct, sea_case, sea_start;
extern  int     repl_case, repl_verify, repl_start;
extern  char    default_name[], save_as_name[], filename[]; 
extern  char    filename_save[], default_path[]; 
extern  char    save_as_path[], search_string[]; 
extern  char    search_string_save[], text[], old_name[]; 
extern  char    new_name[], info_string[], param_filename[]; 
extern  char    sea_str[], sea_str_cfg[], repl_str[], repl_str_cfg[];
extern  char    replby_str[], replby_str_cfg[];


/**********************/ 
/* Start of routines  */ 
/**********************/ 
 

/* main routine for options handling  */ 
 
void handle_options(void) 
{ 
  int         i, a, b, garbage, item_hit; 
  TEDINFO     *ted_info; 

  ted_info = dial_options[TABSIZE].ob_spec.tedinfo; 
  sprintf (text, "%2d", tab_size); 
  strcpy (ted_info->te_ptext, text); 
  ted_info = dial_options[LINELEN].ob_spec.tedinfo; 
  sprintf (text, "%3d", line_len); 
  strcpy (ted_info->te_ptext, text); 
  if (eol_warn)
  {
    objc_change (dial_options, EOLWON, 0, 0, 0, 0, 0, SELECTED, 0); 
    objc_change (dial_options, EOLWOFF, 0, 0, 0, 0, 0, NORMAL, 0); 
  }
  else
  {
    objc_change (dial_options, EOLWOFF, 0, 0, 0, 0, 0, SELECTED, 0); 
    objc_change (dial_options, EOLWON, 0, 0, 0, 0, 0, NORMAL, 0); 
  }
  if (showcr)
  {
    objc_change (dial_options, SHOWCRY, 0, 0, 0, 0, 0, SELECTED, 0); 
    objc_change (dial_options, SHOWCRN, 0, 0, 0, 0, 0, NORMAL, 0); 
  }
  else
  {
    objc_change (dial_options, SHOWCRN, 0, 0, 0, 0, 0, SELECTED, 0); 
    objc_change (dial_options, SHOWCRY, 0, 0, 0, 0, 0, NORMAL, 0); 
  }
  if (showpos)
  {
    objc_change (dial_options, SHOWPOSY, 0, 0, 0, 0, 0, SELECTED, 0); 
    objc_change (dial_options, SHOWPOSN, 0, 0, 0, 0, 0, NORMAL, 0); 
  }
  else
  {
    objc_change (dial_options, SHOWPOSN, 0, 0, 0, 0, 0, SELECTED, 0); 
    objc_change (dial_options, SHOWPOSY, 0, 0, 0, 0, 0, NORMAL, 0); 
  }
  if (backup)
  {
    objc_change (dial_options, BACKYES, 0, 0, 0, 0, 0, SELECTED, 0); 
    objc_change (dial_options, BACKNO, 0, 0, 0, 0, 0, NORMAL, 0); 
  }
  else
  {
    objc_change (dial_options, BACKNO, 0, 0, 0, 0, 0, SELECTED, 0); 
    objc_change (dial_options, BACKYES, 0, 0, 0, 0, 0, NORMAL, 0); 
  }
  if (clocky)
  {
    objc_change (dial_options, CLOCKYES,  
                      0, 0, 0, 0, 0, SELECTED, 0); 
    objc_change (dial_options, CLOCKNO,  
                      0, 0, 0, 0, 0, NORMAL, 0); 
  }
  else
  {
    objc_change (dial_options, CLOCKNO,  
                      0, 0, 0, 0, 0, SELECTED, 0); 
    objc_change (dial_options, CLOCKYES,  
                      0, 0, 0, 0, 0, NORMAL, 0); 
  }
  garbage = TRUE; 
  while (garbage)         /* check for proper setup */ 
  {                             /*  1. while start  */ 
    item_hit = handle_dial(dial_options, 0);
    if (item_hit == OPTCANC) return; 
    garbage  = FALSE; 
    ted_info = dial_options[TABSIZE].ob_spec.tedinfo; 
    a = atoi (ted_info->te_ptext); 
    garbage = (a == 0);
    if (garbage)
    {
      put_message (NO_TAB_SIZE);
      continue;
    }
    ted_info = dial_options[LINELEN].ob_spec.tedinfo; 
    b = atoi (ted_info->te_ptext);
    if (!garbage) garbage = (b == 0);
    if (garbage)
    {
      put_message (NO_LINE_LEN);
      continue;
    }
    tab_size = a;
    line_len = b;
    eol_warn  = 
      (dial_options[EOLWON].ob_state == SELECTED); 
    showcr  = 
      (dial_options[SHOWCRY].ob_state == SELECTED); 
    showpos = 
      (dial_options[SHOWPOSY].ob_state == SELECTED); 
    backup  = 
      (dial_options[BACKYES].ob_state == SELECTED); 
    clocky  = 
      (dial_options[CLOCKYES].ob_state == SELECTED); 
  }                               /* 1. while end */ 
  display_time (clocky);
  for (i = 0; i < WD_MAX_WINDOWS; i++) 
  { 
    if (windows[i].handle != -1)  
    {
      refresh_window (windows[i].handle);
      display_pos (i);
    }
  }
  return;
}


/* end of file options.c  */
