/* 
output.c 
ALICE V 2.0, January 1993
library functions for output handling 
(C) Ralf Kaufmann 1993
used C compiler: Pure C 
*/ 
 
/* Includefiles */ 
 
#define __GEMLIB_OLDNAMES
#include <gem.h> 
#include <stdio.h> 
#include "language.h" 
#include "prvdef.h" 
#include "mewidi.h" 
#include "messages.h" 
#include "scancode.h" 
#include "structs.h" 
#include "rsc.h" 
 
/* References to global data defined elsewhere  */ 
 
extern  short       VDIhandle; 
extern  short       bzeich, hzeich; 
extern  WIND_DATA   windows[]; 
extern  BUFFER      buffers[]; 
extern  OBJECT      *menu_tree; 
extern  OBJECT      *dial_about, *dial_help, *dial_print, *dial_tab; 
extern  int         page_length, left_margin; 
extern  int         tab_size, showcr, showpos; 
extern  int         line_len, eol_warn; 
 
// Function-Prototypes
void clear_window(short w_handle);
short cursor(short w_handle, short mode); 
void display_pos(int w_index);
short get_index_from_handle(short w_handle); 
short HighWord(char *str);
int line_length(int w_index, long number); 
long max(long a, long b);
int own_top_wind(void); 
void put_graphic_char(char c, short x, short y, short w_index); 
void rcopy(short xs, short ys, short ws, short hs, short xd, short yd, 
					 short mode);
int top_wind(void); 
 
/**********************/ 
/* Start of routines  */ 
/**********************/ 
 
 
 
 
/*-----------------------------------------------------------
	Function to output any text 

	w_handle;                       window handle
	*area;                          describes the redraw area
	clear;                          clear before writing ?
-----------------------------------------------------------*/ 
void output(short w_handle, GRECT *area, short clear)
{ 
  long        i, j; 
  int         x, y; 
  short       letter[4], w_index; 
  long        start_x, start_y, end_y; 
  LINE_DESC   *p; 
 
  w_index = get_index_from_handle (w_handle); 
  if (windows[w_index].head == NULL) return; 			/* nothing there   */ 
                                              		/* to be displayed */ 
  if (clear) 
  { 
//    ALLWHITE((area->g_x),(area->g_y),(area->g_w),(area->g_h)); 
		clear_window(w_handle); 
  } 

  start_x = windows[w_index].doc_x / bzeich;  			/* char offset in doc */ 
  start_y = windows[w_index].doc_y / hzeich;  			/* line offset in doc */ 
  end_y   = start_y + (windows[w_index].work.g_h / hzeich); 
 
  if (start_y > windows[w_index].doc_length) return; 
                                         				/* no line to be displayed */ 
  area->g_h += area->g_y; 
  area->g_w += area->g_x; 
  area->g_x -= bzeich; 
  area->g_y -= hzeich; 
 
  for (i = 0, p = windows[w_index].head; i < start_y; i++) 
  { 
    p = p->next; 
  } 
 
  for (y = windows[w_index].work.g_y, i = start_y; 
       y < windows[w_index].work.g_y + windows[w_index].work.g_h && 
       p && i < end_y; 
       y += hzeich, p = p->next, i++) 
  {                                                 /* 1. for start */ 
    for (x = windows[w_index].work.g_x, j = start_x; 
         x < windows[w_index].work.g_x + windows[w_index].work.g_w && 
         j < p->len; 
         x += bzeich, j++) 
    {                                               /* 2. for start */ 
      if (y >= area->g_y && x >= area->g_x && 
          y <= area->g_h && x <= area->g_w )
      {                                             /* 1. if start  */ 
        																				/* block marks required ? */ 
        if  (!windows[w_index].block.set) 
        { 
          put_graphic_char (p->line[j], x, y, w_index);		/* output char  */ 
          continue; 
        } 
        if ((i + 1 < windows[w_index].block.cs_y) || 
            (i + 1 > windows[w_index].block.ce_y)) 
        { 
          put_graphic_char(p->line[j], x, y, w_index);		/* output char  */ 
          continue; 
        } 
        letter[0] = x; 
        letter[1] = y; 
        letter[2] = x + bzeich - 1; 
        letter[3] = y + hzeich - 1; 
				vsf_color(VDIhandle, LWHITE); 
        if ((i + 1 >  windows[w_index].block.cs_y) && 
            (i + 1 <  windows[w_index].block.ce_y)) 
        {
          v_bar(VDIhandle, letter); 
          vswr_mode(VDIhandle, MD_TRANS);     /* write mode is transparent  */ 
        } 
        else if ((i + 1 == windows[w_index].block.cs_y) && 
                 (i + 1 == windows[w_index].block.ce_y) && 
                 (j + 1 >= windows[w_index].block.cs_x) && 
                 (j + 1 <  windows[w_index].block.ce_x)) 
        { 
          v_bar(VDIhandle, letter); 
          vswr_mode(VDIhandle, MD_TRANS);     /* write mode is transparent  */ 
        } 
        else if ((i + 1 == windows[w_index].block.cs_y) && 
                 (i + 1 <  windows[w_index].block.ce_y) && 
                 (j + 1 >= windows[w_index].block.cs_x)) 
        { 
          v_bar(VDIhandle, letter); 
          vswr_mode(VDIhandle, MD_TRANS);     /* write mode is transparent  */ 
        } 
        else if ((i + 1 == windows[w_index].block.ce_y) && 
                 (i + 1 >  windows[w_index].block.cs_y) && 
                 (j + 1 <  windows[w_index].block.ce_x)) 
        { 
          v_bar(VDIhandle, letter); 
          vswr_mode(VDIhandle, MD_TRANS);     /* write mode is transparent  */ 
        } 
        else if ((i + 1 == windows[w_index].block.ce_y) && 
                 (j + 1 == windows[w_index].block.ce_x)) 
        { 
          vswr_mode(VDIhandle, MD_REPLACE); /* write mode is back to replace */ 
        } 
        
        put_graphic_char(p->line[j], x, y, w_index);			/* output char  */ 
      }                                             			/* 1. if end    */ 
    }                                               			/* 2. for end   */ 
  } 
  graf_mouse(M_OFF, NULL);
  if (w_handle == top_wind()) cursor(w_handle, 1); 
  graf_mouse(M_ON, NULL);
  MOUSE_IS_ARROW; 
  vswr_mode(VDIhandle, MD_REPLACE);  		/* write mode is back to replace */ 
}                                           /* end function output  */ 
 
 
 
/*------------------------------------------------------------
	Display char c at position x, y
------------------------------------------------------------*/ 
void put_graphic_char(char c, short x, short y, short w_index)
{ 
  static char one_char[2] = { ' ', '\0'}; 
 
  one_char[0] = c; 
  if (one_char[0] == '\n' && w_index != -1) 
  {
    if (showcr)  
    {
      one_char[0] = EOL_SYMBOL;
    }
    else
    {
      one_char[0] = ' ';
    }
  }

  v_gtext(VDIhandle,              		/* here's where the output  */ 
          x, y,         							/* eventually finds its way */ 
          one_char);                 	/* to the screen            */ 
}                               /* end function put_graphic_char  */ 

/*---------------------------------------------------
	Draw cursor
---------------------------------------------------*/
inline void draw_cursor(short x, short y, short mode) 
{
	short pxy[4];
	
	pxy[0] = x - 1;
	pxy[1] = y;
	pxy[2] = x;
	pxy[3] = y + hzeich - 1;
	vsf_interior(VDIhandle, FIS_SOLID);
	if 			(!mode)				vsf_color(VDIhandle, WHITE);
	else if (mode == 1)		vsf_color(VDIhandle, G_BLUE);
	v_bar(VDIhandle, pxy);
}

/*-------------------------------------- 
	Display or hide text cursor
--------------------------------------*/
short cursor(short w_handle, short mode)
{ 
  short cursor_visible, help, start_x, start_y, x = 0, y = 0, w_index, 
  			wind_w, wind_h; 

  if (!own_top_wind()) return (TRUE);
  if (top_wind() != w_handle) return (TRUE); 
  w_index = get_index_from_handle(w_handle); 
 
  start_x = windows[w_index].doc_x / bzeich; 
  start_y = windows[w_index].doc_y / hzeich; 
  wind_w  = windows[w_index].work.g_w / bzeich; 
                                					/* number of letters in width ... */ 
  wind_h  = windows[w_index].work.g_h / hzeich; 
                                                	/* ... and heigth */ 
  cursor_visible = TRUE; 
  if ((windows[w_index].curs_y < start_y + 1)       || 
      (windows[w_index].curs_y > start_y + wind_h)  || 
      (windows[w_index].curs_x < start_x + 1)       || 
      (windows[w_index].curs_x > start_x + wind_w)) 
  { 
    cursor_visible = FALSE; 
  } 

  if (!cursor_visible)	return (TRUE);
//  if (mode != windows[w_index].curs_vis) 	// Wegen im Sichtbaren suchen
//  { 
    help  = (int) (windows[w_index].curs_x - 1 - start_x) * bzeich; 
    help  = (int) max ((long) help, (long) 0); 
    x     = windows[w_index].work.g_x + help; 
    help  = (int) (windows[w_index].curs_y - 1 - start_y) * hzeich; 
    help  = (int) max ((long) help, (long) 0); 
    y     = windows[w_index].work.g_y + help; 
//    INVERT(x, y, (bzeich-1), (hzeich-1));    		/* display/hide cursor  */ 
		draw_cursor(x, y, mode); 
    windows[w_index].curs_vis ^= 1;           	/* invert the flag  */ 
//  }
									 	
  if (showpos && windows[w_index].curs_vis) 	display_pos(w_index);
  if ((mode != 1) || (top_wind() != w_handle)) 	return (TRUE);

  if (eol_warn == TRUE && 
  		line_length(w_index, windows[w_index].curs_y) > line_len)
  {
    BELL;
  }
  return (TRUE); 
}                                       /* end of function cursor */ 
 

 
/*--------------------------------------
	Display cursor position in info line                    
--------------------------------------*/
void display_pos(int w_index)
{
  int i, p;

  if (dial_print[HEADYES].ob_state == SELECTED)
  {
    p = page_length - 2;
  }
  else
  {
    p = page_length;
  }
  i = (int)((windows[w_index].curs_y - 1)/ p);
  i++;
  if (showpos)
  {
# ifdef english
    sprintf (windows[w_index].info_line, 
              " Page %d, Line %ld (%ld), Column %ld",
              i, 
              windows[w_index].curs_y - (i-1)*p,
              windows[w_index].curs_y, 
              windows[w_index].curs_x);
# endif
# ifdef german
    sprintf (windows[w_index].info_line, 
              " Seite %d, Zeile %ld (%ld), Spalte %ld",
              i,
              windows[w_index].curs_y - (i-1)*p, 
              windows[w_index].curs_y, 
              windows[w_index].curs_x);
# endif
# ifdef french
    sprintf (windows[w_index].info_line, 
              " Page %d, Ligne %ld (%ld), Colonne %ld",
              i,
              windows[w_index].curs_y - (i-1)*p, 
              windows[w_index].curs_y, 
              windows[w_index].curs_x);
# endif
  }
  else
  {
    strcpy (windows[w_index].info_line, "");
  }
  wind_set(windows[w_index].handle, WF_INFO,   /* set info line */ 
           HighWord(windows[w_index].info_line), 
           (short)windows[w_index].info_line, 0, 0);
}

/* end of file output.c  */ 
 
