/* 
slider.c 
ALICE V 2.0, January 1993
library functions for slider handling 
(C) Ralf Kaufmann 1993
used C compiler: Pure C 
*/ 
 
/* Includefiles */ 
 
#include <gem.h> 
#include <stdio.h> 
#include "language.h" 
#include "prvdef.h" 
#include "mewidi.h" 
#include "messages.h" 
#include "scancode.h" 
#include "structs.h" 
#include "rsc.h" 
 
 
/* References to global data defined elsewhere  */ 
 
extern  short     ap_id, VDIhandle; 
extern  short     bzeich, hzeich; 
extern  int       page_length, left_margin; 
extern  OBJECT    *menu_tree; 
extern  OBJECT    *dial_about, *dial_help, *dial_print, *dial_tab; 
extern  OBJECT    *dial_search, *dial_replace; 
extern  int       tab_size; 
 
extern  WIND_DATA windows []; 
extern  BUFFER    buffers []; 

// Funktionsprototypen
short cursor(short w_handle, short mode); 
short do_redraw(short w_handle, short x, short y, short w, short h, 
								short clear); 
short full_redraw(short w_handle, short clear); 
short get_index_from_handle(short w_handle); 
long max(long a, long b);
long min(long a, long b);
void rcopy(short xs, short ys, short ws, short hs, short xd, short yd, 
					 short mode); 

 
/**********************/ 
/* Start of routines  */ 
/**********************/ 
 
 
 
 
void set_slider_size(short w_handle) 
{ 
  short	dummy[4], h_size, v_size, w_index; 
  long  h, v;                 /* longs required for multiplication  */ 
 
 
  w_index = get_index_from_handle (w_handle); 
 
  /* get the visible part of the document */ 
 
  if (windows[w_index].doc_x + windows[w_index].work.g_w - WD_WIPO <= 
      windows[w_index].doc_width) 
  { 
    h = windows[w_index].work.g_w - WD_WIPO; 
  } 
  else 
  { 
    h = windows[w_index].doc_width - windows[w_index].doc_x; 
  } 
  h *= WD_HSLIDE_SIZE_MAX; 
  if (windows[w_index].doc_width != 0) 
  { 
      h /= windows[w_index].doc_width; 
  } 
  h_size = (int) h; 
  if (windows[w_index].doc_width == 0) h_size = WD_HSLIDE_SIZE_MAX; 
  h_size = (int) max ((long) h_size, (long) 0); 
  h_size = (int) min ((long) h_size, (long) WD_HSLIDE_SIZE_MAX); 
 
  /* get the visible part of the document */ 
 
  if (windows[w_index].doc_y + windows[w_index].work.g_h - WD_HEPO <= 
      windows[w_index].doc_length) 
  { 
    v = windows[w_index].work.g_h - WD_HEPO; 
  } 
  else 
  { 
    v = windows[w_index].doc_length - windows[w_index].doc_y; 
  } 
  v *= WD_VSLIDE_SIZE_MAX; 
  if (windows[w_index].doc_length != 0) 
  { 
    v /= windows[w_index].doc_length; 
  } 
  v_size = (int) v; 
  if (windows[w_index].doc_length == 0) v_size = WD_VSLIDE_SIZE_MAX; 
  v_size = (int) max ((long) v_size, (long) 0); 
  v_size = (int) min ((long) v_size, (long) WD_VSLIDE_SIZE_MAX); 
 
  /* set (and save) the new values  */ 
 
  if (h_size != windows[w_index].hslide_size) 
  { 
    wind_set(w_handle, WF_HSLSIZE, h_size, 0, 0, 0); 
    wind_get(w_handle, WF_HSLSIZE, &windows[w_index].hslide_size, 
             &dummy[0], &dummy[1], &dummy[2]); 
  } 
  if (v_size != windows[w_index].vslide_size) 
  { 
    wind_set(w_handle, WF_VSLSIZE, v_size, 0, 0, 0); 
    wind_get(w_handle, WF_VSLSIZE, &windows[w_index].vslide_size, 
             &dummy[0], &dummy[1], &dummy[2]); 
  } 
  return; 
}                             /* end function set_slider_side */ 
 
 
 
 
void set_slider_pos(short w_handle) 
{ 
  short 	x_pos, y_pos, w_index, dummy[4]; 
  size_t  a, b; 

  x_pos = y_pos = 0; 
  a     = b     = 0; 
 
  w_index = get_index_from_handle (w_handle); 
  if (windows[w_index].doc_x + windows[w_index].work.g_w - WD_WIPO >= 
            windows[w_index].doc_width) 
  { 
    x_pos = WD_HSLIDE_POS_MAX; 
  } 
  else 
  { 
    a = windows[w_index].doc_x; 
    if (a == 0) 
    { 
      x_pos = 0; 
    } 
    else 
    { 
      a *= WD_HSLIDE_POS_MAX; 
      b = windows[w_index].doc_width - 
            (windows[w_index].work.g_w - WD_WIPO); 
      x_pos = (int) (a/b); 
    } 
  }                                           /* 1. if end  */ 
  if (windows[w_index].doc_y + windows[w_index].work.g_h - WD_HEPO >= 
            windows[w_index].doc_length) 
  { 
    y_pos = WD_VSLIDE_POS_MAX; 
  } 
  else 
  { 
    a = windows[w_index].doc_y; 
    if (a == 0) 
    { 
      y_pos = 0; 
    } 
    else 
    { 
      a *= WD_VSLIDE_POS_MAX; 
      b = windows[w_index].doc_length - 
            (windows[w_index].work.g_h - WD_HEPO); 
      y_pos = (int) (a/b); 
    } 
  }                                                /* 1. if end  */ 
  /* set (and save) the new values  */ 
  if (x_pos != windows[w_index].hslide_pos) 
  { 
    wind_set(w_handle, WF_HSLIDE, x_pos, 0, 0, 0); 
    wind_get(w_handle, WF_HSLIDE, &windows[w_index].hslide_pos, 
             &dummy[0], &dummy[1], &dummy[2]); 
  } 
  if (y_pos != windows[w_index].vslide_pos) 
  { 
    wind_set(w_handle, WF_VSLIDE, y_pos, 0, 0, 0); 
    wind_get(w_handle, WF_VSLIDE, &windows[w_index].vslide_pos, 
             &dummy[0], &dummy[1], &dummy[2]); 
  } 
  return; 
}                             /* end function set_slider_pos  */ 
 
 
 
 
/* Slidermanager  */ 
 
int scroll_wind(int w_handle, int what) 
//int   w_handle, what; 
{ 
  long  a, b; 
  int   w_index, redraw, slider/*, top_handle, dummy[4]*/; 
  short top_handle, dummy[4]; 
  int   send_buffer[MSG_BUFFER];
  int   background;   
 
  redraw  = FALSE; 
  slider  = FALSE; 
  background = FALSE;
  w_index = get_index_from_handle (w_handle); 
  wind_update (BEG_UPDATE); 
  graf_mouse(M_OFF, NULL);
  cursor (w_handle, 0); 
 
  top_handle = w_handle;
  wind_get (top_handle, WF_TOP, &top_handle, 
            &dummy[0], &dummy[1], &dummy[2]); 
  if (top_handle != w_handle) 
  {
    background = TRUE;
    slider = TRUE;
  }

  switch (what) 
  {                                            /* 1. switch start  */ 
    case  W_UPPAGE: 
                    windows[w_index].doc_y -= 
                    windows[w_index].work.g_h - WD_HEPO - 
                     windows[w_index].scroll_y; /* overlapping line */ 
                    windows[w_index].doc_y = 
                      max ((long) windows[w_index].doc_y, (long) 0); 
 
                    slider = TRUE; 
                    redraw = TRUE; 
                    break; 
 
    case  W_DNPAGE: 
                    windows[w_index].doc_y += 
                    windows[w_index].work.g_h - WD_HEPO - 
                     windows[w_index].scroll_y; /* overlapping line */ 
                    a = windows[w_index].doc_length - 
                        (windows[w_index].work.g_h - WD_HEPO); 
                    a = max (a, 0); 
                    b = windows[w_index].doc_y; 
                    windows[w_index].doc_y = min (a, b); 
                    slider = TRUE; 
                    redraw = TRUE; 
                    break; 
 
    case  W_UPLINE: 
                    if (windows[w_index].doc_y == 0) break; 
                    windows[w_index].doc_y -= 
                    windows[w_index].scroll_y; 
                    windows[w_index].doc_y = 
                      max ((long) windows[w_index].doc_y, (long) 0); 
 
                    if (background) break;
                    
                    /* perform a quick update of the screen */ 
 
                    rcopy (windows[w_index].work.g_x, 
                           windows[w_index].work.g_y, 
                           windows[w_index].work.g_w - 1, 
                           windows[w_index].work.g_h - hzeich - 1, 
                           windows[w_index].work.g_x, 
                           windows[w_index].work.g_y + hzeich, 
                           S_ONLY); 
                    do_redraw (w_handle, 
                               windows[w_index].work.g_x, 
                               windows[w_index].work.g_y, 
                               windows[w_index].work.g_w, 
                               hzeich, TRUE); 
                    slider = TRUE; 
                    break; 
 
    case  W_DNLINE: 
                    if (windows[w_index].doc_y >= 
                        windows[w_index].doc_length - 
                        (windows[w_index].work.g_h - WD_HEPO)) break; 
                    windows[w_index].doc_y += 
                    windows[w_index].scroll_y; 
                    a = windows[w_index].doc_length - 
                        (windows[w_index].work.g_h - WD_HEPO); 
                    a = max (a, 0); 
                    b = windows[w_index].doc_y; 
                    windows[w_index].doc_y = min (a, b); 
 
                    if (background) break;

                    /* perform a quick update of the screen */ 
 
                    rcopy (windows[w_index].work.g_x, 
                           windows[w_index].work.g_y + hzeich, 
                           windows[w_index].work.g_w - 1, 
                           windows[w_index].work.g_h - hzeich - 1, 
                           windows[w_index].work.g_x, 
                           windows[w_index].work.g_y, 
                           S_ONLY); 
                    do_redraw (w_handle, 
                               windows[w_index].work.g_x, 
                               windows[w_index].work.g_y + 
                                windows[w_index].work.g_h - hzeich, 
                               windows[w_index].work.g_w, 
                               hzeich, TRUE); 
                    slider = TRUE; 
                    break; 
 
    case  W_LFPAGE: 
                    windows[w_index].doc_x -= 
                    windows[w_index].work.g_w - WD_WIPO - 
                    windows[w_index].scroll_x; /* overlapping "line" */
                    windows[w_index].doc_x = 
                      max ((long) windows[w_index].doc_x, (long) 0); 
 
                    slider = TRUE; 
                    redraw = TRUE; 
                    break; 
 
    case  W_RTPAGE: 
                    windows[w_index].doc_x += 
                    windows[w_index].work.g_w - WD_WIPO - 
                    windows[w_index].scroll_x; /* overlapping "line" */
                    a = windows[w_index].doc_width - 
                        (windows[w_index].work.g_w - WD_WIPO); 
                    a = max (a, 0); 
                    b = windows[w_index].doc_x; 
                    windows[w_index].doc_x = min (a, b); 
                    slider = TRUE; 
                    redraw = TRUE; 
                    break; 
 
    case  W_LFLINE: 
                    if (windows[w_index].doc_x == 0) break; 
                    windows[w_index].doc_x -= 
                    windows[w_index].scroll_x; 
                    windows[w_index].doc_x = 
                      max ((long) windows[w_index].doc_x, (long) 0); 
 
                    if (background) break;

                    /* perform a quick update of the screen */ 
 
                    rcopy (windows[w_index].work.g_x, 
                           windows[w_index].work.g_y, 
                           windows[w_index].work.g_w - bzeich - 1, 
                           windows[w_index].work.g_h - 1, 
                           windows[w_index].work.g_x + bzeich, 
                           windows[w_index].work.g_y, 
                           S_ONLY); 
                    do_redraw (w_handle, 
                               windows[w_index].work.g_x, 
                               windows[w_index].work.g_y, 
                               bzeich, 
                               windows[w_index].work.g_h, TRUE); 
                    slider = TRUE; 
                    break; 
 
    case  W_RTLINE: 
                    if (windows[w_index].doc_x >= 
                        windows[w_index].doc_width - 
                        (windows[w_index].work.g_w - WD_WIPO)) break; 
                    windows[w_index].doc_x += 
                    windows[w_index].scroll_x; 
                    a = windows[w_index].doc_width - 
                        (windows[w_index].work.g_w - WD_WIPO); 
                    a = max (a, 0); 
                    b = windows[w_index].doc_x; 
                    windows[w_index].doc_x = min (a, b); 
 
                    if (background) break;

                    /* perform a quick update of the screen */ 
 
                    rcopy (windows[w_index].work.g_x + bzeich, 
                           windows[w_index].work.g_y, 
                           windows[w_index].work.g_w - bzeich - 1, 
                           windows[w_index].work.g_h - 1, 
                           windows[w_index].work.g_x, 
                           windows[w_index].work.g_y, 
                           S_ONLY); 
                    do_redraw (w_handle, 
                               windows[w_index].work.g_x + 
                                windows[w_index].work.g_w - bzeich, 
                               windows[w_index].work.g_y, 
                               bzeich, 
                               windows[w_index].work.g_h, TRUE); 
                    slider = TRUE; 
                    break; 
 
    default       : 
                    break; 
 
  }                                              /* 1. switch end  */ 
  if (slider) 
  { 
    set_slider_size (w_handle); 
    set_slider_pos  (w_handle); 
  }
  if (background)
  {
      /* send a full redraw message  */ 
    send_buffer[0] = WM_REDRAW; 
    send_buffer[1] = ap_id; 
    send_buffer[2] = 0; 
    send_buffer[3] = w_handle; 
    send_buffer[4] = 
     windows[w_index].work.g_x; 
    send_buffer[5] = 
     windows[w_index].work.g_y; 
    send_buffer[6] = 
     windows[w_index].work.g_w; 
    send_buffer[7] = 
     windows[w_index].work.g_h; 
    appl_write(ap_id, (short)sizeof(send_buffer), send_buffer); 
    redraw = FALSE;
  } 
  if (redraw) full_redraw (w_handle, TRUE); 
  cursor (w_handle, 1); 
  graf_mouse(M_ON, NULL);
  wind_update (END_UPDATE); 
  return (TRUE); 
}                                       /* end function scroll_wind */ 
 
 
 
 
/* calculate the horizontal document position   */ 
 
int wind_hslide(int w_handle, int newpos) 
//int   w_handle, newpos; 
{ 
  int   w_index/*, dummy[4]*/; 
	short dummy[4]; 
  long  a, b, c; 
 
 
  w_index = get_index_from_handle (w_handle); 
 
  a = windows[w_index].doc_width - 
       (windows[w_index].work.g_w - WD_WIPO); 
  a = max (a, 0); 
  b = (long) newpos * (long) a; 
  b /= WD_HSLIDE_POS_MAX; 
  c = b; 
  c = max (c, 0); 
  c = min (c, a); 
 
  windows[w_index].doc_x = c - c % bzeich;  /* truncate if required */ 
 
  wind_set (w_handle, WF_HSLIDE, newpos, 0, 0, 0); 
 
  wind_get (w_handle, WF_HSLIDE, &windows[w_index].hslide_pos, 
            &dummy[0], &dummy[1], &dummy[2]); 
  return (TRUE); 
}                                       /* end function wind_hslide */ 
 
 
 
 
/* calculate the vertical document position */ 
 
int wind_vslide(int w_handle, int newpos) 
//int   w_handle, newpos; 
{ 
  int   w_index/*, dummy[4]*/; 
  short dummy[4]; 
  long  a, b, c; 
 
 
  w_index = get_index_from_handle (w_handle); 
 
  a = windows[w_index].doc_length - 
       (windows[w_index].work.g_h - WD_HEPO); 
  a = max (a, 0); 
  b = (long) newpos * a; 
  b /= WD_VSLIDE_POS_MAX; 
  c = b; 
  c = max (c, 0); 
  c = min (c, a); 
 
  windows[w_index].doc_y = c - c % hzeich; /* truncate if required */ 
 
  wind_set (w_handle, WF_VSLIDE, newpos, 0, 0, 0); 
 
  wind_get (w_handle, WF_VSLIDE, &windows[w_index].vslide_pos, 
            &dummy[0], &dummy[1], &dummy[2]); 
  return (TRUE); 
}                                      /* end function wind_vslide */ 
 
 
/* end of file slider.c */ 
 
